@@ -61,7 +61,7 @@ def _deriva(funzione, x0, parametri, precisione=0.00000001):
             n-=10
         derivata_x0.append(derivata)
 
-    return derivata_x0
+    return np.array(derivata_x0)
 
 
 def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,sigma_y,
@@ -77,8 +77,9 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     x_chi=0,
     y_chi=0,
     post_decimal_chi=0,
+    post_decimal_params=None,
     log_to_file=False,
-    nolatex=False,
+    debug_mode=False,#disattiva latex e manda i risultati in console
     path=".",
     xscale="linear",
     yscale="linear",
@@ -99,11 +100,13 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     style_residui={"fmt" : "."},
     res_line_color="navy",
     pad_percent=0.10,
+    tratteggio=True,
+    spessore_zero_line=1,
     precisione_curvatura=1001,
     foglio="Foglio1",
     funzioni_modifica=[None, None],
-    residui_ratio=[4, 1],
-    fig_dimension=(6.24136,4.4),
+    residui_ratio=[4.236067977500615, 1],#golden ratio
+    fig_dimension=(6.24136,4.4), #pdf page latex dimension
     estensione_grafico=".pdf"
 ):
     """
@@ -132,7 +135,8 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     popt_names_latex : list
         elenco dei nomi dei parametri ottimali che vengono ricavati dal fit
         verranno usati come contenuto della cella del titolo di una tabella latex contenente i valori numerici dei parametri
-        devono essere scritti in Latex
+        devono essere scritti in Latex, la stringa deve essere composta da una parentesi graffa contenete il nome del parametro seguito
+        da una quadra contenente l'unità di misure [\si ...]
 
     variable_x, variable_y, sigma_y : str
         ogni parametro contine il valore dell'header della colonna del foglio excel da cui prendere i dati
@@ -164,18 +168,20 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
         se True allora sul grafico viene mostrato il valore del chi quadro ridotto,
         se False viene mostrato il chi quadro e i gradi di linertà (default: True)
 
-
     x_chi, y_chi : int, optional
         coordinate del punto in cui scrivere il valore del chi quadro e il numero di gradi di libertà (default: 0, 0 --> primo punto del plot)
 
     post_decimal_chi : int, optional
         numero di cifre dopo la virgola con cui scrivere il valore del chi quadro (default: 0)
 
+    post_decimal_params: list of int, optional
+        numero di cifre dopo la virgola con cui scrivere i parametri di fit nella tabella, se None allora tutte le cifre vengono esplicitate (default: None)
+
     log_to_file : bool, optional
         if True le informazioni sulla bontà del grafico sono messe in un file di testo. If False le informazioni possono essere lette in console. (default: False)
 
-    nolatex : bool, optional
-        if True il grafico è disegnato senza compilare in latex
+    debug_mode : bool, optional
+        if True il grafico è disegnato senza compilare in latex e le info sono scritte in console (default: False)
 
     path : str, optional
         path directory di partenza (default: ".")
@@ -198,6 +204,9 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     pad_pecent : float, optional
         dimensione percentuale dello spazio del grafico in più rispetto al minimo necessario a contenere i dati (default: 0.10)
 
+    tratteggio : bool, optional
+        se True indica che i valori aggiunti da "pad_percent" vengano rappresentati prolungando la linea del grafico con un tratteggio (default: True)
+
     precisione_curvatura : int, optional
         numero di punti utilizzato per il disegno della curva (default: 1001)
 
@@ -326,7 +335,7 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
         popt = output.beta
         pcov = output.cov_beta
 
-
+    popt_sigma=np.sqrt(pcov.diagonal())
 
     #chi^2
     
@@ -337,6 +346,13 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     gradi_di_liberta=len(t)-len(popt)
     chi_rid=chisq/gradi_di_liberta    
  
+    if debug_mode:
+        print("debug mode: ON")
+        print("parametri ottimali di fit: ")
+        print(popt)
+        print("errori associati: ")
+        print(popt_sigma)
+
     #valutazione grafico
     sopra_fit=0
     sotto_fit=0
@@ -348,14 +364,25 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
             sotto_fit+=1
         if np.absolute(element_y-f_fit(element_t,*popt))>element_dy:
             lontano_fit+=1
-    p_value=stats.distributions.chi2.sf(chi_rid, 1)
+    p_value=stats.distributions.chi2.sf(chi_rid, gradi_di_liberta)
 
-    frase_log=[]
+    frase_log=["","","analisi del fit:",""]
     frase_log.append("Numero di punti sopra alla funzione di best fit: "+str(sopra_fit)+"\n"+"Numero di punti sotto alla funzione di best fit: "+str(sotto_fit))
     frase_log.append("(expected: " + str(len(t)/2)+")")
-    frase_log.append("Numero di punti che distano dal grafico n>1 barre di errore: "+str(lontano_fit)+" (expected-ipotesi  errori gaussiani: "+str(0.32 *len(t))+")")
-    frase_log.append("la probabilità di ottenere un valore più estremo è di: "+p_value)
-    if log_to_file:
+    frase_log.append("")
+    frase_log.append("Numero di punti che distano dal grafico n>1 barre di errore: "+str(lontano_fit)+" (expected-nell'ipotesi di errori gaussiani-: "+str(0.32 *len(t))+")")
+    frase_log.append("")
+    frase_log.append("")
+    frase_log.append("Il chi quadro vale: "+str(chisq))
+    frase_log.append("per un numero di gdl/dof di: "+str(gradi_di_liberta))
+    frase_log.append("")
+    if p_value<0.5:
+        frase_log.append("probability of obtaining a more extreme value is: "+str(p_value))
+    else:
+        frase_log.append("probability of obtaining a more extreme value is: "+str(1-p_value))
+    frase_log.append("probability of 22 heads on a coin flip is: 2.38e-07 (2.87 valore gaussiano per >5 sigma)")
+
+    if log_to_file and not debug_mode:
         with open(path+"\\"+file_result+"_fit_log.txt", 'w') as fit_log:
             fit_log.write("\n".join(frase_log))
             fit_log.close()
@@ -365,24 +392,32 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
 
 
     #scrittura file: *_tabella.tex con i risultati di fit (longtable)
+            
+    if not debug_mode:
+        colonne="c"*len(popt)*2 # colonne centrate in numero pari al numero di parametri
+        line_begin=r"\begin{longtable}{"+colonne+r"}"
 
-    colonne="c"*len(popt) # colonne centrate in numero pari al numero di parametri
-    line_begin=r"\begin{longtable}{"+colonne+r"}"
-    table_header="&".join(popt_names_latex)+r"\\"
-    longtable_header=[r"\toprule",table_header ,r"\midrule", r"\endfirsthead", r"\toprule",table_header ,r"\midrule",r"\endhead", r"midrule", r"\multicolumn{3}{r}{{continua a pagina seguente}} \\", r"\midrule", r"\endfoot", r"\bottomrule", r"\endlastfoot"]
-    string_popt=[str(i_opt) for i_opt in popt] #numeri convertiti in stringa
-    data_table=r" & ".join(string_popt)+r" \\"
-    line_end=r"\end{longtable}"
+        popt_names_latex=[r"\(\hat"+str(e)+r"\) & \(\sigma"+str(e)+"\)" for e in popt_names_latex]
+        table_header="&".join(popt_names_latex)+r"\\"
+        longtable_header=[r"\toprule",table_header ,r"\midrule", r"\endfirsthead", r"\toprule",table_header ,r"\midrule",r"\endhead", r"\midrule", r"\multicolumn{"+str(len(popt)*2)+r"}{r}{{continua a pagina seguente}} \\", r"\midrule", r"\endfoot", r"\bottomrule", r"\endlastfoot"]
+        if post_decimal_params:
+            string_popt=[str(round(i_opt, post_decimal_params_i)) +"&" + str(round(sigma_i_opt, post_decimal_params_i)) for i_opt,sigma_i_opt, post_decimal_params_i in zip(popt,popt_sigma,post_decimal_params)] #numeri convertiti in stringa
+        else:
+            string_popt=[str(i_opt) +"&" + str(sigma_i_opt) for i_opt,sigma_i_opt in zip(popt,popt_sigma)] #numeri convertiti in stringa
 
-    tabella=[] #creazione tabella completa
-    tabella.append(line_begin)
-    tabella.extend(longtable_header)
-    tabella.append(data_table)
-    tabella.append(line_end)
-    
-    with open(path+"\\"+file_result+"_tabella.tex", 'w') as tabella_tex: #scrittura file .tex
-        tabella_tex.write("\n".join(tabella))
-        tabella_tex.close()
+        data_table=r" & ".join(string_popt)+r" \\"
+        
+        line_end=r"\end{longtable}"
+
+        tabella=[] #creazione tabella completa
+        tabella.append(line_begin)
+        tabella.extend(longtable_header)
+        tabella.append(data_table)
+        tabella.append(line_end)
+        
+        with open(path+"\\"+file_result+"_tabella.tex", 'w') as tabella_tex: #scrittura file .tex
+            tabella_tex.write("\n".join(tabella))
+            tabella_tex.close()
 
     #disegno grafico 
     plt.rcParams["figure.figsize"] = fig_dimension #dimensioni impostate come richiesto (utile per corretta dimensione dei caratteri)
@@ -393,7 +428,7 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
         if min(t)-pad>0 or min(t)<0:
             lim_inf=min(t)-pad
         else: 
-            lim_inf=0
+            lim_inf=min(t)*(1-pad_percent)
     if not lim_sup:
         if max(t)+pad<0 or max(t)>0:
             lim_sup=max(t)+pad
@@ -408,14 +443,15 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
 
     #TODO: nolatex mode for faster debug
     #Latex parameters
-    plt.rcParams.update({"text.usetex" : True,   #utilizza latex nella compilazione del grafico
-        "font.family": "computer modern",
-        "text.latex.preamble": "\n".join([ # plots will use this preamble
-        r"\usepackage[utf8]{inputenc}",
-        r"\usepackage[T1]{fontenc}",
-        r"\usepackage{siunitx}",
-        r"\usepackage{amsmath}",
-        ])})
+    if not debug_mode:
+        plt.rcParams.update({"text.usetex" : True,   #utilizza latex nella compilazione del grafico
+            "font.family": "computer modern",
+            "text.latex.preamble": "\n".join([ # plots will use this preamble
+            r"\usepackage[utf8]{inputenc}",
+            r"\usepackage[T1]{fontenc}",
+            r"\usepackage{siunitx}",
+            r"\usepackage{amsmath}",
+            ])})
 
     #default label impostati a nome della ccolonna di excel usata per estrarre i dati   
     if not xlabel:
@@ -439,9 +475,10 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     ax1.grid(**griglia_fit)
     ##plot TODO: verificare che funzioni con dx=0
     ax1.errorbar(t, y, dy, dx, **style_fit)
-    ax1.plot(a1, f_fit(a1, *popt), linestyle="dashed", color=fit_line_color)
     ax1.plot(a2, f_fit(a2, *popt), color=fit_line_color)
-    ax1.plot(a3, f_fit(a3, *popt), linestyle="dashed", color=fit_line_color)
+    if tratteggio:
+        ax1.plot(a1, f_fit(a1, *popt), linestyle="dashed", color=fit_line_color)
+        ax1.plot(a3, f_fit(a3, *popt), linestyle="dashed", color=fit_line_color)
     ##adjust limits
     ax1.set_xlim(min(a1),max(a3))
 
@@ -455,9 +492,9 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     ##plot
     y_res=y-f_fit(np.array(t), *popt)
     ax2.errorbar(t, y_res, dy, **style_residui)
-    ax2.plot(a1, a1-a1, color=res_line_color) #evidenzia la linea dello zero
-    ax2.plot(a2, a2-a2, color=res_line_color)
-    ax2.plot(a3, a3-a3, color=res_line_color)
+    ax2.plot(a1, a1-a1, color=res_line_color, linewidth=spessore_zero_line)
+    ax2.plot(a2, a2-a2, color=res_line_color, linewidth=spessore_zero_line)
+    ax2.plot(a3, a3-a3, color=res_line_color, linewidth=spessore_zero_line)
     ##adjust ylim for x axis symmetry
     low, high = ax2.get_ylim() 
     bound = max(abs(low), abs(high)) # find the new limits
@@ -472,11 +509,14 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
         y_chi=y[0]+0.1*(max(y)-min(y))
 
     if chi:
-        if ridotto:
-            ax1.text(x_chi,y_chi,r"$$\frac{\chi^2}{\nu}="+str(round(chi_rid,post_decimal_chi))+r"$$")
+        if not debug_mode:
+            if ridotto:
+                ax1.text(x_chi,y_chi,r"$$\frac{\chi^2}{\nu}="+str(round(chi_rid,post_decimal_chi))+r"$$")
+            else:
+                ax1.text(x_chi,y_chi,r"$$\chi^2/\nu="+str(round(chisq,post_decimal_chi))+r"/"+str(gradi_di_liberta)+r"$$")
         else:
-            ax1.text(x_chi,y_chi,r"$$\chi^2 / \nu}="+str(round(chi,post_decimal_chi))+r" / "+gradi_di_liberta+r"$$")
-            
+            ax1.text(x_chi,y_chi,"il chi quadro è qui")
+                    
 
     fig1.supxlabel(xlabel, y=0.05)
     fig1.supylabel(ylabel, x=0.03)
@@ -505,6 +545,7 @@ def fit(excel_name,file_result,f_fit,popt_names_latex,variabile_x,variabile_y,si
     fig1.canvas.manager.set_window_title("Grafico") 
     fig1.savefig(path+'\\'+file_result+estensione_grafico)
     plt.show()
+    plt.rcParams.update({"text.usetex" : False})
 
 def tabella(excel_name,file_result,
     path=".",
